import unittest
import scanResultUtil
import datetime
import yaml
import os


class TestResultApi(unittest.TestCase):

    def test_calculate_report_mismatch_cvss(self):
        full_report = [{
            "id": "CVE-2021-33910",
            "severity": "HIGH",
            "fix_available": "245.4-4ubuntu3.10",
            "cvss": {
                "v3": 5.5
            },
            "risk": 5
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["high"], 0)
        self.assertEqual(r["target_all"]["medium"], 1)

    def test_calculate_report_no_cvss(self):
        full_report = [{
            "id": "CVE-2021-33910",
            "severity": "HIGH",
            "fix_available": "245.4-4ubuntu3.10",
            "risk": 5
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["high"], 1)
        self.assertEqual(r["target_all"]["medium"], 0)

    def test_calculate_report_cvss_critical(self):
        full_report = [{
            "id": "CVE-2021-0001",
            "cvss": {
                "v3": 9.2
            }
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["critical"], 1)
        self.assertEqual(r["target_all"]["high"], 0)
        self.assertEqual(r["target_all"]["medium"], 0)
        self.assertEqual(r["target_all"]["low"], 0)
        self.assertEqual(r["target_all"]["total"], 1)
        self.assertEqual(r["target_with_fixes"]["critical"], 0)
        self.assertEqual(r["target_with_fixes"]["high"], 0)
        self.assertEqual(r["target_with_fixes"]["medium"], 0)
        self.assertEqual(r["target_with_fixes"]["low"], 0)
        self.assertEqual(r["target_with_fixes"]["unknown"], 0)
        self.assertEqual(r["target_with_fixes"]["total"], 0)

    def test_calculate_report_cvss_high(self):
        full_report = [{
            "id": "CVE-2021-0001",
            "cvss": {
                "v3": 8
            }
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["critical"], 0)
        self.assertEqual(r["target_all"]["high"], 1)
        self.assertEqual(r["target_all"]["medium"], 0)
        self.assertEqual(r["target_all"]["low"], 0)
        self.assertEqual(r["target_all"]["total"], 1)

    def test_calculate_report_cvss_medium(self):
        full_report = [{
            "id": "CVE-2021-0001",
            "cvss": {
                "v3": 5
            }
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["critical"], 0)
        self.assertEqual(r["target_all"]["high"], 0)
        self.assertEqual(r["target_all"]["medium"], 1)
        self.assertEqual(r["target_all"]["low"], 0)
        self.assertEqual(r["target_all"]["total"], 1)

    def test_calculate_report_cvss_low(self):
        full_report = [{
            "id": "CVE-2021-0001",
            "cvss": {
                "v3": 3.2
            }
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["critical"], 0)
        self.assertEqual(r["target_all"]["high"], 0)
        self.assertEqual(r["target_all"]["medium"], 0)
        self.assertEqual(r["target_all"]["low"], 1)
        self.assertEqual(r["target_all"]["total"], 1)

    def test_calculate_report_cvss_all(self):
        full_report = [
            {
                "id": "CVE-2021-0001",
                "cvss": {
                    "v3": 9.3
                }
            },
            {
                "id": "CVE-2021-0002",
                "cvss": {
                    "v3": 7.5
                }
            },
            {
                "id": "CVE-2021-0003",
                "cvss": {
                    "v3": 6.1
                }
            },
            {
                "id": "CVE-2021-0004",
                "cvss": {
                    "v3": 2.9
                }
            }
        ]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["critical"], 1)
        self.assertEqual(r["target_all"]["high"], 1)
        self.assertEqual(r["target_all"]["medium"], 1)
        self.assertEqual(r["target_all"]["low"], 1)
        self.assertEqual(r["target_all"]["total"], 4)

    def test_calculate_report_no_cvss_v3(self):
        full_report = [{
            "id": "CVE-2021-33910",
            "severity": "HIGH",
            "fix_available": "245.4-4ubuntu3.10",
            "cvss": {
                "v2": 3.2
            },
            "risk": 5
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["high"], 1)
        self.assertEqual(r["target_all"]["medium"], 0)
        self.assertEqual(r["target_with_fixes"]["critical"], 0)
        self.assertEqual(r["target_with_fixes"]["high"], 1)
        self.assertEqual(r["target_with_fixes"]["medium"], 0)
        self.assertEqual(r["target_with_fixes"]["low"], 0)
        self.assertEqual(r["target_with_fixes"]["unknown"], 0)
        self.assertEqual(r["target_with_fixes"]["total"], 1)

    def test_calculate_report_no_cvss_no_severity(self):
        full_report = [{
            "id": "CVE-2021-33910",
            "fix_available": "245.4-4ubuntu3.10",
            "risk": 5
        }]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["high"], 0)
        self.assertEqual(r["target_all"]["medium"], 0)
        self.assertEqual(r["target_all"]["unknown"], 1)
        self.assertEqual(r["target_with_fixes"]["critical"], 0)
        self.assertEqual(r["target_with_fixes"]["high"], 0)
        self.assertEqual(r["target_with_fixes"]["medium"], 0)
        self.assertEqual(r["target_with_fixes"]["low"], 0)
        self.assertEqual(r["target_with_fixes"]["unknown"], 1)
        self.assertEqual(r["target_with_fixes"]["total"], 1)

    def test_calculate_report_fixes(self):
        full_report = [
            {
                "id": "CVE-2021-XXXXX",
                "fix_available": "245.4-4ubuntu3.10",
                "risk": 5
            },
            {
                "id": "CVE-2021-XXXXX",
                "fix_available": "",
                "risk": 5
            },
        ]

        r = scanResultUtil.calculate_report(full_report)

        self.assertEqual(r["target_all"]["critical"], 0)
        self.assertEqual(r["target_all"]["high"], 0)
        self.assertEqual(r["target_all"]["medium"], 0)
        self.assertEqual(r["target_all"]["low"], 0)
        self.assertEqual(r["target_all"]["unknown"], 2)
        self.assertEqual(r["target_all"]["total"], 2)
        self.assertEqual(r["target_with_fixes"]["critical"], 0)
        self.assertEqual(r["target_with_fixes"]["high"], 0)
        self.assertEqual(r["target_with_fixes"]["medium"], 0)
        self.assertEqual(r["target_with_fixes"]["low"], 0)
        self.assertEqual(r["target_with_fixes"]["unknown"], 1)
        self.assertEqual(r["target_with_fixes"]["total"], 1)

    def test_calculate_report_baseline_missing_or_empty(self):
        full_report = [{
            "id": "CVE-2021-0001",
            "cvss": {
                "v3": 9.2
            }
        }]

        # no baseline argument
        r = scanResultUtil.calculate_report(full_report)
        self.assertEqual(r["target_all"]["critical"], 1)

        os.environ.update({"BASE_DIFF": "true"})
        scanResultUtil.base_diff_flag = str(os.environ.get("BASE_DIFF")).lower()
        # baseline is empty list
        diffed_report = scanResultUtil.report_diff(full_report, [])
        r = scanResultUtil.calculate_report(diffed_report)
        self.assertEqual(r["target_all"]["critical"], 1)

    def test_calculate_report_baseline_diff(self):
        full_report = [
            {
                "id": "CVE-2021-0001",
                "cvss": {
                    "v3": 9.2
                }
            },
            {
                "id": "CVE-2021-0002",
                "cvss": {
                    "v3": 9.3
                }
            }
        ]

        # baseline contains only 1 CVE in common
        report = [
            {
                "id": "CVE-2021-0001",
                "cvss": {"v3": 9.2}
            }
        ]

        # test with BASE_DIFF flag not set to true i.e. no diff-ing
        os.environ.update({"BASE_DIFF": ""})
        scanResultUtil.base_diff_flag = str(os.environ.get("BASE_DIFF")).lower()
        diffed_report = scanResultUtil.report_diff(full_report, report)
        r = scanResultUtil.calculate_report(diffed_report)
        # There are still 2 items. Statuses should not change since BASE_DIFF is not true
        self.assertEqual(len(diffed_report), 2)
        self.assertEqual(r["target_all"]["critical"], 2)
        self.assertFalse("status" in diffed_report[0])
        self.assertFalse("status" in diffed_report[1])

        # set the BASE_DIFF flag to true
        os.environ.update({"BASE_DIFF": "true"})
        scanResultUtil.base_diff_flag = str(os.environ.get("BASE_DIFF")).lower()
        diffed_report = scanResultUtil.report_diff(full_report, report)
        r = scanResultUtil.calculate_report(diffed_report)
        # There are still 2 items, but one is marked as vuln_status["base"]
        self.assertEqual(len(diffed_report), 2)
        self.assertEqual(r["target_all"]["critical"], 1)
        self.assertEqual(diffed_report[0]["status"], scanResultUtil.vuln_status["base"])
        self.assertFalse("status" in diffed_report[1])

        # baseline contains all CVE from full report
        report = [
            {
                "id": "CVE-2021-0001",
                "cvss": {"v3": 9.2}
            },
            {"id": "CVE-2021-0002", "cvss": {"v3": 9.3}}
        ]
        diffed_report = scanResultUtil.report_diff(full_report, report)
        r = scanResultUtil.calculate_report(diffed_report)
        self.assertEqual(len(diffed_report), 2)
        self.assertEqual(r["target_all"]["critical"], 0)

    def test_show_report(self):
        scanResultUtil.show_report([])

    def test_parse_twistlock(self):
        tl_json = {"results": []}
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r, [])

        tl_json = {"results": [{"vulnerabilities": []}]}
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r, [])

        v = {}
        v["id"] = "CVE-0000-0000"
        v["status"] = "fixed"
        v["cvss"] = 3.7
        v["packageName"] = "org.apache.santuario_xmlsec"
        v["packageVersion"] = "0.0.1"
        tl_json = {"results": [{"vulnerabilities": [v]}]}
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r[0]["id"], v["id"])
        self.assertEqual(r[0]["fix_available"], "fixed")
        self.assertEqual(r[0]["cvss"], {"v3": 3.7})
        self.assertEqual(r[0]["package"], "org.apache.santuario_xmlsec-0.0.1")

        v = {}
        v["cvss"] = 2.3
        v["vector"] = "CVSS:2"
        v["packageName"] = "org.apache.santuario_xmlsec"
        v["packageVersion"] = "0.0.2"
        tl_json = {"results": [{"vulnerabilities": [v]}]}
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r[0]["cvss"], {"v2": 2.3})
        self.assertEqual(r[0]["package"], "org.apache.santuario_xmlsec-0.0.2")

        v = {}
        v["cvss"] = 8.0
        v["vector"] = "CVSS:3"
        tl_json = {"results": [{"vulnerabilities": [v]}]}
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r[0]["cvss"], {"v3": 8.0})

    def test_parse_twistlock_with_binaries(self):
        v = {}
        v["id"] = "PRISMA-2021-0213",
        v["status"] = "fixed in 2.14, 2.13.1, 2.12.6"
        v["cvss"] = 7.5
        v["severity"] = "high"
        v["vector"] = "CVSS:3.1/..."
        v["packageName"] = "com.fasterxml.jackson.core_jackson-databind"
        v["packageVersion"] = "2.12.1"

        p = {
            'type': 'jar',
            'name': 'com.fasterxml.jackson.core_jackson-databind',
            'version': '2.12.1',
            'path': '/mocked/jackson-databind-2.12.1.jar',
            'licenses': ['The Apache Software License, Version 2.0']
        }

        tl_json = {
            "results": [
                {
                    "vulnerabilities": [v],
                    "packages": [p, p],
                }
            ]
        }

        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r[0]["cvss"], {"v3": 7.5})
        self.assertEqual(
            r[0]["paths"],
            [
                "/mocked/jackson-databind-2.12.1.jar",
                "/mocked/jackson-databind-2.12.1.jar"
            ]
        )
        self.assertEqual(len(r), 1)

        v["cvss"] = 7.5
        del v["vector"]
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r[0]["cvss"], {"v3": 7.5})

        del v["cvss"]
        r = scanResultUtil.parse_twistlock(tl_json)
        self.assertEqual(r[0]["cvss"], {"v3": -1})

    def test_analyze_vulns(self):
        v = {}
        v["id"] = "CVE-0000-0000"
        v["cvss"] = {"v3": 9.5}
        v["fix_available"] = ""

        # Test 1. Warning on analyze_vulns()
        scanResultUtil.scan_status["fail"] = False
        scanResultUtil.scan_status["warning"] = False

        normalized = scanResultUtil.normalize_report([v])

        self.assertEqual(scanResultUtil.scan_status["fail"], False)
        self.assertEqual(scanResultUtil.scan_status["warning"], False)

        scanResultUtil.analyze_vulns(normalized)

        self.assertEqual(scanResultUtil.scan_status["fail"], False)
        self.assertEqual(scanResultUtil.scan_status["warning"], True)

        # Test 2. Fail on analyze_vulns()
        v["fix_available"] = "yes"
        scanResultUtil.scan_status["fail"] = False
        scanResultUtil.scan_status["warning"] = False

        normalized = scanResultUtil.normalize_report([v])

        self.assertEqual(scanResultUtil.scan_status["fail"], False)
        self.assertEqual(scanResultUtil.scan_status["warning"], False)

        scanResultUtil.analyze_vulns(normalized)

        scanResultUtil.analyze_vulns([v])
        self.assertEqual(scanResultUtil.scan_status["fail"], True)
        self.assertEqual(scanResultUtil.scan_status["warning"], False)

    def test_normalize_report(self):
        v = {}
        v["id"] = "CVE-0000-0000"

        v["severity"] = "CRITICAL"
        scanResultUtil.scan_status["fail"] = False
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["cvss"]["v3"], 9.01)

        # Normalizing the report should not trigger a fail
        self.assertEqual(scanResultUtil.scan_status["fail"], False)

        v["cvss"] = {"v2": 0, "v3": 0}
        v["severity"] = "CRITICAL"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["cvss"]["v3"], 9.01)

        v["cvss"] = {}
        v["severity"] = "CRITICAL"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["cvss"]["v3"], 9.01)

        v["cvss"] = {"v2": 0, "v3": 0.0}
        v["severity"] = "HIGH"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["cvss"]["v3"], 7.01)

        v["cvss"] = {"v2": 0, "v3": ""}
        v["severity"] = "medium"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["cvss"]["v3"], 4.01)

        v["cvss"] = {"v2": 0, "v3": -1.0}
        v["severity"] = "lOW"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["cvss"]["v3"], 0.11)

        v = {}
        v["id"] = "CVE-2022-26945"
        v["status"] = "fixed"
        v["cvss"] = 0.0
        v["severity"] = "high"
        v["packageName"] = "github.com/hashicorp/go-getter"
        v["packageVersion"] = "v1.5.9"
        p = {}
        p["type"] = "go"
        p["name"] = "github.com/hashicorp/go-getter"
        p["version"] = "v1.5.9"
        p["licenses"] = ["GPL-2.0-only"]
        tl_json = {"results": [{"vulnerabilities": [v], "packages": [p]}]}
        r = scanResultUtil.normalize_report(
            scanResultUtil.parse_twistlock(tl_json)
        )
        # Test CVSS score populated based on the 'severity'
        self.assertEqual(r[0]["cvss"]["v3"], 7.01)
        # Test package type being inferred
        self.assertEqual(r[0]["package_type"], "go")

        v["cvss"] = {}
        v["severity"] = "CRITICAL"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["fix_available"], "")

        v["cvss"] = {}
        v["severity"] = "CRITICAL"
        v["fix_available"] = "open"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["fix_available"], "")

        v = {}
        v["id"] = "CVE-2020-25864"
        v["package_name"] = "github.com/hashicorp/consul"
        v["package_version"] = "v0.0.0-20171026175957-610f3c86a089"
        v["package_type"] = "go-module"
        r = scanResultUtil.normalize_report([v])
        self.assertEqual(r[0]["package_name"], "github.com/hashicorp/consul")
        self.assertEqual(r[0]["package_version"], "v0.0.0-20171026175957-610f3c86a089")
        self.assertEqual(r[0]["package_type"], "go-module")

    def test_slo_medium(self):
        v = {}
        v["id"] = "CVE-0000-0000"
        v["cvss"] = {"v3": 5.5}  # Medium
        v["fix_available"] = "fixed version x.xx"
        scanResultUtil.scan_status["fail"] = False

        # No fail on Medium w/o fixed date
        r = scanResultUtil.normalize_report([v])
        scanResultUtil.analyze_vulns(r)
        self.assertEqual(scanResultUtil.scan_status["fail"], False)

        # No fail on Medium w/ fixed date not over the SLO
        t_now = datetime.datetime.now()
        td = t_now - datetime.timedelta(days=scanResultUtil.SLO['medium'] - 1)
        v["fix_date"] = td.strftime("%Y-%m-%dT%H:%M:%S")
        r = scanResultUtil.normalize_report([v])
        scanResultUtil.analyze_vulns(r)
        self.assertEqual(scanResultUtil.scan_status["fail"], False)

        # fail on Medium w/ fixed date over the SLO
        t_now = datetime.datetime.now()
        td = t_now - datetime.timedelta(days=scanResultUtil.SLO['medium'] + 1)
        v["fix_date"] = td.strftime("%Y-%m-%dT%H:%M:%S")
        r = scanResultUtil.normalize_report([v])
        scanResultUtil.analyze_vulns(r)
        self.assertEqual(scanResultUtil.scan_status["fail"], True)

    def test_slo_low(self):
        v = {}
        v["id"] = "CVE-0000-0000"
        v["cvss"] = {"v3": 1.0}
        v["fix_available"] = "fixed version x.xx"
        scanResultUtil.scan_status["fail"] = False

        # No fail on Low w/o fixed date
        r = scanResultUtil.normalize_report([v])
        scanResultUtil.analyze_vulns(r)
        self.assertEqual(scanResultUtil.scan_status["fail"], False)

        # No fail on Low w/ fixed date not over the SLO
        t_now = datetime.datetime.now()
        td = t_now - datetime.timedelta(days=scanResultUtil.SLO['low'] - 1)
        v["fix_date"] = td.strftime("%Y-%m-%dT%H:%M:%S")
        r = scanResultUtil.normalize_report([v])
        scanResultUtil.analyze_vulns(r)
        self.assertEqual(scanResultUtil.scan_status["fail"], False)

        # fail on Low w/ fixed date over the SLO
        t_now = datetime.datetime.now()
        td = t_now - datetime.timedelta(days=scanResultUtil.SLO['low'] + 1)
        v["fix_date"] = td.strftime("%Y-%m-%dT%H:%M:%S")
        r = scanResultUtil.normalize_report([v])
        scanResultUtil.analyze_vulns(r)
        self.assertEqual(scanResultUtil.scan_status["fail"], True)

    def test_parse_cbctl_with_binaries(self):
        v = {}
        v["id"] = "CVE-0000-0000",
        v["package_name"] = "test_package"
        v["package_version"] = "0.0.1"

        packages = {
            "artifacts": [
                {
                    "id": "0123456789",
                    "name": "test_package",
                    "version": "0.0.1",
                    "locations": [
                        {"path": "/1"},
                        {"path": "/2"},
                    ]
                }
            ]
        }

        pkgs = {}
        r = scanResultUtil.parse_cbctl_binary_vulnerabilities([v], pkgs)
        self.assertEqual(len(r), 1)

        pkgs = {"artifacts": None}
        r = scanResultUtil.parse_cbctl_binary_vulnerabilities([v], pkgs)
        self.assertEqual(len(r), 1)

        with self.assertRaises(Exception):
            # Testing that this throws an error since there is no "path" key
            r[0]["path"]

        r = scanResultUtil.parse_cbctl_binary_vulnerabilities([v], packages)
        self.assertEqual(len(r), 1)
        self.assertEqual(r[0]["paths"], ["/1", "/2"])

    def test_rules_yaml(self):
        self.assertEqual(scanResultUtil.load_rules(), True)

        # Check the exception structure
        for ex in scanResultUtil.rules["exceptions"]:

            self.assertIn("name", ex)
            self.assertIn("description", ex)
            self.assertIn("image", ex)

            # Try parsing the `until` attribute if present
            if "until" in ex:
                until = datetime.datetime.strptime(ex["until"], "%Y-%m-%d")

    def test_is_exception(self):
        v = {
            "id": "CVE-2020-14060",
            "package": "jackson-databind-2.9.6",
            "package_name": "jackson-databind",
            "package_version": "2.9.6",
            "package_type": "java",
            "severity": "HIGH",
            "fix_available": "2.9.10.5",
            "cvss": {
                "v2": 6.8,
                "v3": 8.1
            },
            "paths": ["/test/ehcache-2.10.6.jar"],
            "status": "Fail"
        }
        scanResultUtil.exceptions["by_id"] = ["CVE-2020-14060"]
        self.assertEqual(scanResultUtil.is_exception(v), True)
        scanResultUtil.exceptions["by_id"] = ["Nothing here", "CVE-2020.*"]
        self.assertEqual(scanResultUtil.is_exception(v), True)

        v["id"] = ""
        self.assertEqual(scanResultUtil.is_exception(v), False)
        scanResultUtil.exceptions["by_package"] = [
            {"package": "nothing", "version": "nothing"},
            {"package": ".*jackson-databind.*", "version": "2\.9\..*"}
        ]
        self.assertEqual(scanResultUtil.is_exception(v), True)

        v["package_name"] = ""
        v["package_version"] = ""
        self.assertEqual(scanResultUtil.is_exception(v), False)
        scanResultUtil.exceptions["by_file"] = [
            "/etc/shadow",
            "/test/ehcache-2.10.6.jar"
        ]
        self.assertEqual(scanResultUtil.is_exception(v), True)
        scanResultUtil.exceptions["by_file"] = [
            "/etc/shadow",
            "/test/.*\.jar"
        ]
        self.assertEqual(scanResultUtil.is_exception(v), True)

    def test_is_exception_applicable(self):
        ex = {

            "by_id": [],
        }

        # Check if applicable for this image
        ex["image"] = "noimage"
        self.assertEqual(scanResultUtil.is_exception_applicable(ex, image="psec-test"), False)

        ex["image"] = ".*psec-test.*"
        self.assertEqual(scanResultUtil.is_exception_applicable(ex, image="psec-test"), True)

        # Also test the implicit value for `image` from env var
        os.environ["IMAGE"] = "psec-test"
        ex["image"] = "noimage"
        self.assertEqual(scanResultUtil.is_exception_applicable(ex), False)

        ex["image"] = ".*psec-test.*"
        self.assertEqual(scanResultUtil.is_exception_applicable(ex), True)

        # Check if `until` expiration is properly enforced
        ex["until"] = "1999-12-31"
        self.assertEqual(scanResultUtil.is_exception_applicable(ex, image="psec-test"), False)

        ex["until"] = "2099-12-31"
        self.assertEqual(scanResultUtil.is_exception_applicable(ex, image="psec-test"), True)

        # Not having `until` means the exception does not expire
        del ex["until"]
        self.assertEqual(scanResultUtil.is_exception_applicable(ex, image="psec-test"), True)

    def test_parse_license(self):
        lic = scanResultUtil.parse_license("GPLv2+ and GPLv3+ and LGPLv2+")
        self.assertEqual(lic, ["GPLv2+", "GPLv3+", "LGPLv2+"])

        lic = scanResultUtil.parse_license("LGPLv2.1,GPLv2")
        self.assertEqual(lic, ["LGPLv2.1", "GPLv2"])

        lic = scanResultUtil.parse_license("GPLv2+ or LGPLv3+")
        self.assertEqual(lic, ["GPLv2+", "LGPLv3+"])

        lic = scanResultUtil.parse_license(
            "The Apache Software License, Version 2.0"
        )
        self.assertEqual(lic, ["The Apache Software License, Version 2.0"])

    def test_check_file_reputation_cbctl(self):
        file_reputation = [
            {
                "file_name": "file_name_1",
                "path": "/1"
            },
            {
                "file_name": "file_name_2",
                "path": "/2"
            }
        ]

        scanResultUtil.scan_status["fail"] = False
        scanResultUtil.status_message["fail"][-1] = ""

        # Check clean
        scanResultUtil.check_file_reputation_cbctl([], notify=False)

        self.assertEqual(scanResultUtil.scan_status["fail"], False)
        self.assertEqual(scanResultUtil.status_message["fail"][-1], "")

        # Check with entries
        scanResultUtil.check_file_reputation_cbctl(file_reputation, notify=False)

        self.assertEqual(scanResultUtil.scan_status["fail"], True)
        self.assertNotEqual(scanResultUtil.status_message["fail"][-1], "")

    def test_check_compliance_twistcli(self):
        compliances = [
            {
                "id": 422,
                "title": "Image contains malware",
                "cause": "Found malware xyz"
            },
            {
                "id": 426,
                "title": "Image contains binaries used for crypto mining",
                "cause": "Found binary xyz"
            }
        ]

        scanResultUtil.scan_status["fail"] = False
        scanResultUtil.status_message["fail"][-1] = ""

        # Check clean
        scanResultUtil.check_compliance_twistcli([], notify=False)

        self.assertEqual(scanResultUtil.scan_status["fail"], False)
        self.assertEqual(scanResultUtil.status_message["fail"][-1], "")

        # Check with entries
        scanResultUtil.check_compliance_twistcli(compliances, notify=False)

        self.assertEqual(scanResultUtil.scan_status["fail"], True)
        self.assertNotEqual(scanResultUtil.status_message["fail"][-1], "")

    def test_info_os_version_string(self):
        v_s = scanResultUtil.get_version_from_os_info("Alpine Linux 3.19.1")
        self.assertEqual(v_s, 3.19)

        v_s = scanResultUtil.get_version_from_os_info("Ubuntu 22.04")
        self.assertEqual(v_s, 22.04)

        v_s = scanResultUtil.get_version_from_os_info("VMware Photon OS 3.0")
        self.assertEqual(v_s, 3.0)

        v_s = scanResultUtil.get_version_from_os_info("Fedora Linux 38")
        self.assertEqual(v_s, 38)

        v_s = scanResultUtil.get_version_from_os_info("Lorem Ipsum")
        self.assertEqual(v_s, -1.0)


if __name__ == '__main__':
    unittest.main()
